
#include "camera.hpp"

namespace gar3d {

	vec3 Camera::getUpVector(vec3 forward, vec3 right) {
		vec3 ret = glm::cross(right, forward);
		return(glm::length(ret) == 0) ? vec3(0, 0, 0) : glm::normalize(ret);
	}

	void Perspective::apply() {
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(fov, aspect, zmin, zmax);
	}

	void OrbitCamera::apply(InputHandle& input) {
		if (input.mouseButtonHeld(mouse_left)) {
			int x, y;
			input.getMouseMotion(x, y);
			onMouseMotion(x, y);
		}
		pers.apply();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		vec3 eye = getPosition();
		gluLookAt(eye.x, eye.y, eye.z, 0, 0, 0, 0, 1, 0);
	}

	void OrbitCamera::onMouseMotion(GLfloat xrel, GLfloat yrel) {
		lon -= 0.005*xrel;
		lat += 0.005*yrel;
		if (lat < -M_PI / 2 + 0.001)
			lat = -M_PI / 2 + 0.001;
		if (lat > M_PI / 2 - 0.001)
			lat = M_PI / 2 - 0.001;
	}

	vec3 OrbitCamera::getPosition() {
		return dist*vec3(sin(lon)*cos(lat), sin(lat), cos(lon)*cos(lat));
	}


	void PanCamera::apply(InputHandle& input) {
		if (input.mouseButtonHeld(mouse_left)) {
			int x, y;
			input.getMouseMotion(x, y);
			onMouseMotion(x, y);
		}
		pers.apply();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		vec3 tar = position + direction;
		vec3 up = getUpVector(direction, -x);
		gluLookAt(position.x, position.y, position.z, tar.x, tar.y, tar.z, up.x, up.y, up.z);
	}

	void PanCamera::onMouseMotion(GLfloat xrel, GLfloat yrel) {
		position += x * xrel / GLfloat(50);
		position += y * yrel / GLfloat(50/sqrt(2));
		if (position.x < minbounds.x) {
			position.x = minbounds.x;
		}
		if (position.y < minbounds.y) {
			position.y = minbounds.y;
		}
		if (position.z < minbounds.z) {
			position.z = minbounds.z;
		}
		if (position.x > maxbounds.x) {
			position.x = maxbounds.x;
		}
		if (position.y > maxbounds.y) {
			position.y = maxbounds.y;
		}
		if (position.z > maxbounds.z) {
			position.z = maxbounds.z;
		}
	}

	vec3 PanCamera::getPosition() {
		return position;
	}


	void FreeCamera::apply(InputHandle& input) {
		pers.apply();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(position.x, position.y, position.z, target.x, target.y, target.z, 0, 1, 0);
	}

	vec3 FreeCamera::getPosition() {
		return position;
	}

}