
#include "Time.h"

TimeKeeper::TimeKeeper() : TimeKeeper(Mode::Continuous) {}
TimeKeeper::TimeKeeper(int m) : _start(Timepoint()),
_stepstart(Timepoint(_start)),
_prevstepstart(Timepoint(_start)) {
	_mode = static_cast<Mode>(m);
	_dirty = 0b11;
}
Timepoint TimeKeeper::Now() {
	switch (_mode) {
	case Mode::Discrete:
		return _stepstart;
	case Mode::Continuous:
	default:
		return Timepoint(CLK::now());
	}
}
double TimeKeeper::Now(TimeUnit unit) {
	Timepoint tp = Now();
	switch (unit) {
	case TimeUnit::Nanosecond:
		return tp.Nanoseconds();
	case TimeUnit::Microsecond:
		return tp.Microseconds();
	case TimeUnit::Millisecond:
		return tp.Milliseconds();
	case TimeUnit::Second:
		return tp.Seconds();
	case TimeUnit::Minute:
		return tp.Minutes();
	case TimeUnit::Hour:
		return tp.Hours();
	case TimeUnit::Day:
		return tp.Days();
	default:
		throw tp;
	}
}
double TimeKeeper::DeltaTime() {
	if (_dirty & Dirtyflags::deltat) {
		_deltat = _stepstart - _prevstepstart;
	}
	return _deltat;
}
Duration TimeKeeper::Runtime() {
	return (Now() - _start);
}
double TimeKeeper::StepsPerSecond() {
	if (_dirty & Dirtyflags::sps) {
		_sps = 1.0 / DeltaTime();
	}
	return _deltat;
}
void TimeKeeper::apply_timestep() {
	_prevstepstart = _stepstart;
	_stepstart = Timepoint();
}
void TimeKeeper::HangUntil(Timepoint tp) {
	while (Timepoint() < tp);
}
void TimeKeeper::HangUntil(bool(*_test)(void)) {
	while (!_test());
}
template <class T>
void TimeKeeper::HangUntil(bool(*_test)(T), T arg) {
	while (!_test(arg));
}
void TimeKeeper::HangFor(Duration d) {
	Timepoint end = Timepoint() + d;
	while (Timepoint() < end);
}