
#include "Timepoint.h"

Timepoint::Timepoint()
	: Timepoint(CLK::now()) {}
Timepoint::Timepoint(T_P tp)
	: Timepoint(tp.time_since_epoch().count()) {}
double Timepoint::Nanoseconds() {
	return _nanos;
}
double Timepoint::Microseconds() {
	if (dirtyflags & _flags::micro) {
		_micros = _nanos / 1e3;
		dirtyflags |= (~_flags::micro);
	}
	return _micros;
}
double Timepoint::Milliseconds() {
	if (dirtyflags & _flags::milli) {
		_millis = _nanos / 1e6;
		dirtyflags |= (~_flags::milli);
	}
	return _millis;
}
double Timepoint::Seconds() {
	if (dirtyflags & _flags::sec) {
		_secs = _nanos / 1e9;
		dirtyflags |= (~_flags::sec);
	}
	return _secs;
}
double Timepoint::Minutes() {
	if (dirtyflags & _flags::min) {
		_mins = _nanos / 6e10;
		dirtyflags |= (~_flags::min);
	}
	return _mins;
}
double Timepoint::Hours() {
	if (dirtyflags & _flags::hour) {
		_hours = _nanos / 3.6e12;
		dirtyflags |= (~_flags::hour);
	}
	return _hours;
}
double Timepoint::Days() {
	if (dirtyflags & _flags::day) {
		_days = _nanos / 8.64e13;
		dirtyflags |= (~_flags::day);
	}
	return _days;
}
Timepoint::Timepoint(double nanos) {
	_nanos = nanos;
	dirtyflags = 0x7e;
}
Duration operator -(Timepoint tp1, Timepoint tp0) {
	return Duration(tp1.Nanoseconds() - tp0.Nanoseconds(), TimeUnit::Nanosecond);
}
Timepoint operator +(Timepoint tp, Duration d) {
	return Timepoint(tp.Nanoseconds() + d.Nanoseconds());
}
Timepoint operator -(Timepoint tp, Duration d) {
	return Timepoint(tp.Nanoseconds() - d.Nanoseconds());
}
Timepoint Timepoint::operator +=(Duration d) {
	_nanos += d.Nanoseconds();
	dirtyflags = 0x7e;
	return *this;
}
Timepoint Timepoint::operator -=(Duration d) {
	_nanos -= d.Nanoseconds();
	dirtyflags = 0x7e;
	return *this;
}
bool operator ==(Timepoint tp1, Timepoint tp2) {
	return tp1.Nanoseconds() == tp2.Nanoseconds();
}
bool operator !=(Timepoint tp1, Timepoint tp2) {
	return tp1.Nanoseconds() != tp2.Nanoseconds();
}
bool operator >=(Timepoint tp1, Timepoint tp2) {
	return tp1.Nanoseconds() >= tp2.Nanoseconds();
}
bool operator <=(Timepoint tp1, Timepoint tp2) {
	return tp1.Nanoseconds() <= tp2.Nanoseconds();
}
bool operator >(Timepoint tp1, Timepoint tp2) {
	return tp1.Nanoseconds() > tp2.Nanoseconds();
}
bool operator <(Timepoint tp1, Timepoint tp2) {
	return tp1.Nanoseconds() < tp2.Nanoseconds();
}
