#ifndef CAMERA_HPP
#define CAMERA_HPP

#define _USE_MATH_DEFINES
#include <cmath>
#include "SDL_GLM_INCLUDES.hpp"
#include "InputHandle.hpp"

namespace gar3d {

	using glm::vec3;

	class Perspective {
	public:
		Perspective(float fov = 90, float aspect = 1,
			float zmin = 0.1, float zmax = 50) :
			fov(fov), aspect(aspect), zmin(zmin), zmax(zmax) {}
		void apply();
	protected:
		float fov, aspect, zmin, zmax;
	};

	class Camera {
		friend class Gar3D;
	public:
		virtual void apply(InputHandle& input) = 0;
		virtual vec3 getPosition() = 0;
	protected:
		vec3 minbounds;
		vec3 maxbounds;
		vec3 getUpVector(vec3 forward, vec3 right);
		Camera() : pers() {}
		Camera(Perspective p) : pers(p) {}
		Perspective pers;
	};




	class OrbitCamera : public Camera {
	public:
		OrbitCamera(float dist = 1, float lat = 0, float lon = 0,
			Perspective pers = Perspective()) : Camera(pers),
			dist(dist), lat(lat), lon(lon) {}
		void apply(InputHandle&);
		void onMouseMotion(GLfloat xrel, GLfloat yrel);
		vec3 getPosition();
	protected:
		float dist, lat, lon;
	};

	class PanCamera : public Camera {
	public:
		PanCamera() : PanCamera(vec3(0, 2, 4), vec3(-1, 0, 0), vec3(0, 0, -1), vec3(0, -1, -2), Perspective(30, 16 / 9., 1, 20)) {}
		PanCamera(vec3 pos, vec3 xdir, vec3 ydir, vec3 facedir, Perspective p)
			: position(pos),
			x(glm::normalize(xdir)),
			y(glm::normalize(ydir)), 
			direction(glm::normalize(facedir)) { pers = p; }
		void apply(InputHandle&);
		void onMouseMotion(GLfloat xrel, GLfloat yrel);
		vec3 getPosition();
	protected:
		vec3 position;
		vec3 x;
		vec3 y;
		vec3 direction;
	};

	class FreeCamera : public Camera {
	public:
		FreeCamera() : FreeCamera(vec3(0, 2, 4), vec3(0, 0, 0), Perspective(30, 16 / 9., 1, 20)) {}
		FreeCamera(vec3 pos, vec3 lookAt, Perspective p) : position(pos), target(lookAt) { pers = p; }
		void apply(InputHandle&);
		vec3 position;
		vec3 target;
		vec3 getPosition();
	};


}

#endif
